from contextlib import nullcontext
from io import BytesIO, StringIO

import pytest

from json_stream_rs_tokenizer import RustTokenizer


@pytest.mark.parametrize(
    "buffering,expected_str_cursor_pos,expected_bytes_cursor_pos",
    [
        (1, 7, 10),  # unbuffered
        (2000, 14, 17),  # large buffer
        (-1, 14, 17),  # don't care => should choose large buf
    ],
)
def test_buffering_cursor_pos(
    buffering,
    expected_str_cursor_pos,
    expected_bytes_cursor_pos,
    to_bytes_or_str_buf,
):
    """
    Test that buffering setting is respected by checking cursor pos.
    """
    buf = to_bytes_or_str_buf('[ "äöµ", "a" ]')
    tokenizer = RustTokenizer(buf, buffering=buffering, correct_cursor=False)
    for kind, val in tokenizer:
        if val == "äöµ":
            break
    else:
        assert False, "didn't find expected list elem for some reason"
    if isinstance(buf, StringIO):
        assert buf.tell() == expected_str_cursor_pos
    elif isinstance(buf, BytesIO):
        assert buf.tell() == expected_bytes_cursor_pos
    else:
        assert False, "what"


@pytest.mark.parametrize(
    "buffering",
    [
        1,  # unbuffered
        2000,  # large buffer
        -1,  # don't care => should choose large buf
    ],
)
def test_buffering_cursor_pos_with_correct_cursor_enforcement(
    buffering, to_bytes_or_str_buf
):
    """
    Test that buffering setting plays nicely with correct_cursor.

    Could have gone into either the overconsumption tests or here, but here
    should be fine.
    """
    buf = to_bytes_or_str_buf('[ "äöµ", "a" ]')
    raises_ctx = (
        pytest.raises(ValueError)
        if not buf.seekable() and buffering > 1
        else nullcontext()
    )
    with raises_ctx:
        tokenizer = RustTokenizer(
            buf, buffering=buffering, correct_cursor=True
        )
    if raises_ctx != nullcontext:
        return  # nothing else to check
    for kind, val in tokenizer:
        if val == "äöµ":
            break
    else:
        assert False, "didn't find expected list elem for some reason"
    if isinstance(buf, StringIO):
        assert buf.tell() == 7
    elif isinstance(buf, BytesIO):
        assert buf.tell() == 10
    else:
        assert False, "what"



from sys import stderr

import typer
from si_prefix import si_parse

from . import app


cli = typer.Typer()


@cli.command()
def run(
    json_bytes: str = typer.Option(
        default="2 M",
        metavar="SI_NUMBER",
        help=(
            "Size of random JSON to generate in bytes. "
            "Use of SI prefixes (e.g. 1k = 1000) is possible."
        ),
    )
):
    try:
        json_bytes = si_parse(json_bytes)
    except Exception:
        print(
            f"Could not parse as SI-prefixed number of bytes: '{json_bytes}'",
            file=stderr,
        )
        raise typer.Exit(code=1)
    app.main(json_bytes)


def main():
    return cli()



try:
    from .cli import main
except ImportError as e:
    raise ImportError(
        "benchmark dependencies not installed, please consult the README"
    ) from e

if __name__ == "__main__":
    exit(main())



from io import StringIO
from json_stream_rs_tokenizer import load


def test_integers():
    assert list(load(StringIO("[123]"))) == [123]
    assert list(load(StringIO("[123e3]"))) == [123e3]
    assert list(load(StringIO("[123E3]"))) == [123E3]
    assert list(load(StringIO("[-123E3]"))) == [-123E3]



"""
Regression test for overconsumption of stream contents past the end of a doc:
https://github.com/smheidrich/py-json-stream-rs-tokenizer/issues/47
"""
from io import BytesIO, StringIO

import pytest

from json_stream_rs_tokenizer import RustTokenizer, load


@pytest.fixture()
def to_bytes_or_str(to_bytes_or_str_buf):
    xio = to_bytes_or_str_buf("")
    if isinstance(xio, StringIO):
        return lambda s: s
    elif isinstance(xio, BytesIO):
        return lambda s: s.encode("utf-8")
    else:
        assert False


# this test requires a version of json-stream that supports park_cursor()
@pytest.mark.xfail(reason="json-stream chicken and egg problem")
@pytest.mark.parametrize(
    "s,expected_cursor_pos",
    [
        ('{ "a": 1 } { "b": 2 }', 10),
        ('{"a": 1} { "b": 2 }', 8),
        ('{"a":1} { "b": 2 }', 7),
        ('{ "a":1, "b": 2, "c": 3, "d": 4, "xyz": 99999 } { "b": 2 }', 47),
        ('{ "a": [1, 2, 3, 4, 5 ], "d": 4, "xyz": 99999 } { "b": 2 }', 47),
    ],
)
def test_overconsumption_load_ends_at_doc_end(
    s, expected_cursor_pos, to_bytes_or_str_buf
):
    buf = to_bytes_or_str_buf(s)
    list(load(buf))
    assert buf.tell() == expected_cursor_pos


@pytest.mark.parametrize(
    "s,expected_str_cursor_pos,expected_bytes_cursor_pos",
    [
        ('{ "a": 1 } | { "b": 2 }', 10, 10),
        ('{"a": 1} | { "b": 2 }', 8, 8),
        ('{"a":1} | { "b": 2 }', 7, 7),
        ('{ "a":1, "b": 2, "c": 3, "d": 4, "xyz": 9 } | { "b": 2 }', 43, 43),
        ('{ "æ": [1, 2, 3, 4, 5 ], "ð": 4, "xyz": 9 } | { "b": 2 }', 43, 45),
    ],
)
def test_overconsumption_park_cursor_skip_3_chars_and_continue(
    s, expected_str_cursor_pos, expected_bytes_cursor_pos, to_bytes_or_str_buf
):
    """
    Principal regression test for overconsumption.
    """
    buf = to_bytes_or_str_buf(s)
    tokenizer = RustTokenizer(buf)
    for kind, val in tokenizer:
        if val == "}":
            break
    tokenizer.park_cursor()
    if isinstance(buf, StringIO):
        assert buf.tell() == expected_str_cursor_pos
    elif isinstance(buf, BytesIO):
        assert buf.tell() == expected_bytes_cursor_pos
    else:
        assert False, "what"
    buf.read(3)  # skip ahead 3 chars
    assert "".join(str(val) for kind, val in tokenizer) == "{b:2}"


@pytest.mark.parametrize(
    "s",
    [
        ('{ "a": 1 } | { "b": 2 }'),
        ('{"a": 1} | { "b": 2 }'),
        ('{"a":1} | { "b": 2 }'),
        ('{ "a":1, "b": 2, "c": 3, "d": 4, "xyz": 9 } | { "b": 2 }'),
        ('{ "æ": [1, 2, 3, 4, 5 ], "ð": 4, "xyz": 9 } | { "b": 2 }'),
    ],
)
def test_correct_cursor_disabled(s, to_bytes_or_str_buf, to_bytes_or_str):
    """
    Test that overconsumption occurs when cursor correctness is not requested.

    In that case, we do readahead buffering regardless of seekability to
    maximize performance.

    Also tests that the remainder is correct in this case, which allows people
    to build their own workarounds for the issue in Python if they so choose.
    """
    buf = to_bytes_or_str_buf(s)
    tokenizer = RustTokenizer(buf, correct_cursor=False)
    for kind, val in tokenizer:
        if val == "}":
            break
    assert len(buf.read()) == 0  # EOF => readahead buf read everything
    assert tokenizer.remainder == to_bytes_or_str(s.split("}", maxsplit=1)[1])
